-------------  unregister\register ----------
local reg, unreg, UNREG, events = common.RegisterEventHandler, common.UnRegisterEventHandler, common.UnRegisterEvent, {}

common.RegisterEventHandler = function( func, event )
	if events[event] then	
		if not events[event][func] then
			events[event][func] = func					
			reg( func, event )		
		end	
	else	
		events[event] = {}		
		events[event][func] = func				
		reg( func, event )	
	end
end

common.UnRegisterEventHandler = function( func, event )
	if events[event] then	
		if events[event][func] then
			events[event][func] = nil		
			unreg( func, event )		
		end		
	end
end

common.UnRegisterEvent = function( event )
	if events[event] then	
		UNREG( event )		
		events[event] = nil					
	end
end

-------------  unregister\register ----------
local RegisterEvent, UnRegisterEvent, LogInfo, RegisterReactionHandler = common.RegisterEventHandler, common.UnRegisterEventHandler, common.LogInfo, common.RegisterReactionHandler
--------------------------------------------------------------------------------
local wtMainPanel = mainForm:GetChildChecked( "MainPanel", false )
local wtAdditionalPanel = mainForm:GetChildChecked( "AdditionalPanel", false )
local wtBuffInternal = wtMainPanel:GetChildChecked( "BuffInternal", false )

local lowPriorityBuff = 30*24*60*60*1000 --   ,     .   .
local formatStack		= "<header color='0xFFFFFFFF' fontname='AllodsWest' fontsize='15' alignx='right' outline='1'><r name='stack'/></header>"
local formatBuffTimer	= "<header color='0xFFFFFFFF' fontname='AllodsWest' fontsize='15' alignx='left' outline='1'><r name='timer'/></header>"
local formatSeconds		= "<header color='0xFFFFFFFF' fontname='AllodsWest' fontsize='10' alignx='left' outline='1'><r name='secondsTimer'/></header>"

local wtMainPanelDNDId = 897
local wtAdditionalPanelDNDId = 895

local Texts = {
	[ "rus" ] = {" ", " ", " ", " ", " ", " "," ", " ", "", "  "},
	["fra"] = {"Force de Givre", "Pouvoir des elements", "Force de Foudre", "Force de Feu", "Eclair", "Aurore boreale", "Atout de tonnerre", "Gehenne Brulante", "Aspiration de l'obscurite"},
	[ "eng" ] = {"Frost Force", "Fire Force", "Lightning Force", "Conduit", "Northern Lights", "Thundering Advantage", "Magical Entropy", "Stone Wall", "Stone Barrier", "Reflections"}
}

local localization = "rus"
local L = nil
local additionalBuffs = {}
local avatarBuffs = {}
local showAdditional = true
local isMage = false
--------------------------------------------------------------------------------
local function StackCount( ability, stackCount, buffs )
	if stackCount > 1 then							
		buffs[ ability ]:GetChildChecked( "Stack", false ):SetVal( "stack", common.FormatInt( stackCount, "%d" ) )
		buffs[ ability ]:GetChildChecked( "Stack", false ):Show( true )
	else
		buffs[ ability ]:GetChildChecked( "Stack", false ):Show( false )
	end
end
--------------------------------------------------------------------------------
local function BuffTime( ability, remainingMs, buffs )
	if remainingMs > 0 then
		local seconds = math.floor( remainingMs / 1000 )
		local buffTimerString = seconds.."s"
		local buffSecondsTimerString = ""
		if seconds >= 60 then		
			local minutes = math.floor( seconds / 60 )
			seconds = math.fmod( seconds, 60 ) 
			buffTimerString = minutes.."m"
			buffSecondsTimerString = seconds.."s"
			if minutes >= 60 then
				local hours = math.floor( minutes / 60 )
				minutes = math.fmod( minutes, 60 ) 
				buffTimerString = hours.."h"
				buffSecondsTimerString = minutes.."m"
				if hours >= 24 then
					local days = math.floor( hours / 24 )
					hours = math.fmod( hours, 24 )
					buffTimerString = days.."d"
					buffSecondsTimerString = hours.."h"
				end
			end
		end
		buffs[ ability ]:GetChildChecked( "BuffTimer", false):SetVal( "timer", userMods.ToWString( buffTimerString ) )
		buffs[ ability ]:GetChildChecked( "BuffSecondsTimer", false ):SetVal( "secondsTimer", userMods.ToWString( buffSecondsTimerString ) )
	end
end
--------------------------------------------------------------------------------
local function CreateBuff( buff, ability, buffs, panel )	
	buffs[ ability ] = mainForm:CreateWidgetByDesc( wtBuffInternal:GetWidgetDesc() )
	if buff.texture			then buffs[ ability ]:GetChildChecked( "Texture", false ):SetBackgroundTexture( buff.texture ) end
	if buff.isStackable		then buffs[ ability ]:GetChildChecked( "Stack", false ):SetFormat( userMods.ToWString( formatStack ) ) end	
	if buff.durationMs > 0	then
		buffs[ ability ]:GetChildChecked( "BuffTimer", false ):SetFormat( userMods.ToWString( formatBuffTimer ) )
		buffs[ ability ]:GetChildChecked( "BuffSecondsTimer", false ):SetFormat( userMods.ToWString( formatSeconds ) )
		buffs[ ability ]:GetChildChecked( "BuffSecondsTimer", false ):PlayFadeEffect( 0.0, 0.65, 1, EA_MONOTONOUS_INCREASE )
	end
	StackCount( ability, buff.stackCount, buffs )
	BuffTime( ability, buff.remainingMs, buffs )
	panel:AddChild( buffs[ ability ] )
	buffs[ ability ]:Show( true )
end
--------------------------------------------------------------------------------
local function removeBuffs(list, buffs, widgetBuffs)
	for _, ability in pairs(list) do
		if not buffs[ ability ] then
			if widgetBuffs[ ability ] then
				widgetBuffs[ ability ]:Show( false )
				widgetBuffs[ ability ]:DestroyWidget()
				widgetBuffs[ ability ] = nil
			end	
		end
	end
end
--------------------------------------------------------------------------------
local function OnBuffsChanged( eventParams )
	local avatarId = avatar.GetId()
	if eventParams == nil or eventParams.objectId == avatarId then
		local totalBuffCount = object.GetBuffCount( avatarId )
		local buffCount = 0	
		local additionalBuffCount = 0
		local currentBuffs = {}
		local currentAdditionalBuffs = {}
		local sortTable = {}
		local activeBuffs = object.GetBuffs( avatarId )
		for index = 0, totalBuffCount - 1 do
			local buff = object.GetBuffInfo( activeBuffs[ index ] )			
			
			if buff and buff.isPositive then
				if isMage then
					for _, ability in pairs(L) do
						if userMods.FromWString( buff.name ) == ability then
							if not avatarBuffs[ ability ] then CreateBuff( buff, ability, avatarBuffs, wtMainPanel) end
							currentBuffs[ ability ] = ability
							buffCount = buffCount + 1
						end
					end
				end
				if showAdditional then
					for _, ability in pairs(buffsList) do
						if userMods.FromWString( buff.name ) == ability then
							if not additionalBuffs[ ability ] then CreateBuff( buff, ability, additionalBuffs, wtAdditionalPanel) end
							currentAdditionalBuffs[ ability ] = ability
							additionalBuffCount = additionalBuffCount + 1
							sortTable[ additionalBuffCount ] = {}
							sortTable[ additionalBuffCount ].ability = ability
							if buff.remainingMs > 0 then sortTable[ additionalBuffCount ].remainingMs = buff.remainingMs
							else sortTable[ additionalBuffCount ].remainingMs = lowPriorityBuff
							end				
						end
					end
				end
			end			
		end
		removeBuffs(L, currentBuffs, avatarBuffs)
		if showAdditional then
			removeBuffs(buffsList, currentAdditionalBuffs, additionalBuffs)
		end

		table.sort( sortTable, function( a, b ) return a.remainingMs < b.remainingMs end ) --   1 :  sortTable[ 0 ]  
		if buffCount > 0 then
			local index = 0
			for _, ability in pairs(L) do
				if avatarBuffs[ability] ~= nil then
					local placement = avatarBuffs[ ability ]:GetPlacementPlain()
					placement.posY = 15
					placement.posX = 15 + ( 40 * index )
					avatarBuffs[ ability ]:SetPlacementPlain( placement )
					---
					placement = wtMainPanel:GetPlacementPlain()
					placement.sizeX = 70 + ( 40 * ( index ) )
					wtMainPanel:SetPlacementPlain( placement )
					index = index + 1
				end
				
			end
			wtMainPanel:Show( true )
		else
			wtMainPanel:Show( false )
		end	
		
		if additionalBuffCount > 0 then
			for index = 0, additionalBuffCount - 1 do
				local placement = additionalBuffs[ sortTable[ index + 1 ].ability ]:GetPlacementPlain()
				placement.posY = 15
				placement.posX = 15 + ( 40 * index )
				additionalBuffs[ sortTable[ index + 1 ].ability ]:SetPlacementPlain( placement )
				---
				placement = wtAdditionalPanel:GetPlacementPlain()
				placement.sizeX = 70 + ( 40 * ( index ) )
				wtAdditionalPanel:SetPlacementPlain( placement )
				wtAdditionalPanel:Show( true )
			end	
		else
			wtAdditionalPanel:Show( false )
		end
	end
end
--------------------------------------------------------------------------------
local function SecondTimer()
	local avatarId = avatar.GetId()
	local totalBuffCount = object.GetBuffCount( avatarId )
	local activeBuffs = object.GetBuffs( avatarId )
	for index = 0, totalBuffCount - 1 do
		local buff = object.GetBuffInfo( activeBuffs[ index ] )			
		if buff and buff.isPositive then
			local ability = userMods.FromWString( buff.name )
			if isMage then
				if avatarBuffs[ ability ] then
					BuffTime( ability, buff.remainingMs, avatarBuffs )
					StackCount( ability, buff.stackCount, avatarBuffs )
				end
			end
			if showAdditional then			
				if additionalBuffs[ ability ] then
					BuffTime( ability, buff.remainingMs, additionalBuffs )
					StackCount( ability, buff.stackCount, additionalBuffs )
				end
			end
		end
	end
end
--------------------------------------------------------------------------------
local function OnAMAddonInfoRequest( eventParams )
	if eventParams.target == common.GetAddonName() then
		userMods.SendEvent( "SCRIPT_ADDON_INFO_RESPONSE", {
			sender = common.GetAddonName(),
			desc = "  .",
		})
	end
end
--------------------------------------------------------------------------------
local function OnAMAddonMemRequest( eventParams )
	if eventParams.target == common.GetAddonName() then
		userMods.SendEvent( "SCRIPT_ADDON_MEM_RESPONSE", { 
			sender = eventParams.target,
			memUsage = gcinfo() 
		})
	end
end
-- AO game Localization detection by SLA. Version 2011-02-10.
local function GetGameLocalization()
	local B = cartographer.GetMapBlocks()
	local T = { rus="\203\232\227\224", eng="Holy Land", ger="Heiliges Land",
	fra="Terre Sacr\233e", br="Terra Sagrada", jpn="\131\74\131\106\131\65" }
	for b in pairs(B) do for l,t in pairs(T) do
	if userMods.FromWString( cartographer.GetMapBlockInfo(B[b]).name ) == t
	then return l end; end; end; return "eng"
end
--------------------------------------------------------------------------------
local function WidgetsInit()
	wtMainPanel:Show( false )
	wtAdditionalPanel:Show( false)
	wtBuffInternal:Show( false )
	DnD:Init( wtMainPanelDNDId, wtMainPanel, wtMainPanel, true, true, { -8, -8, -8, -8 } )
	DnD:Init( wtAdditionalPanelDNDId, wtAdditionalPanel, wtAdditionalPanel, true, true, { -8, -8, -8, -8 } )
end
--------------------------------------------------------------------------------
local function OnReactionExecute()
	if showAdditional then
		showAdditional = false
		UnRegisterEvent( OnBuffsChanged,	"EVENT_OBJECT_BUFFS_CHANGED" )	
		UnRegisterEvent( SecondTimer,		"EVENT_SECOND_TIMER" )		
		for _, v in pairs(additionalBuffs) do
			if additionalBuffs[ v ] then
				additionalBuffs[ v ]:Show( false )
				additionalBuffs[ v ]:DestroyWidget()
				additionalBuffs[ v ] = nil
			end
		end
		additionalBuffs = {}
		wtAdditionalPanel:Show( false)
		RegisterEvent( OnBuffsChanged,	"EVENT_OBJECT_BUFFS_CHANGED" )	
		RegisterEvent( SecondTimer,		"EVENT_SECOND_TIMER" )		
	else
		showAdditional = true
		OnBuffsChanged()
	end
end
--------------------------------------------------------------------------------
-- INITIALIZATION
--------------------------------------------------------------------------------
local function Init()

	if avatar.GetClass() == "MAGE" then
	 isMage = true
	end

	localization = GetGameLocalization()
	L = Texts[ localization ]
	UnRegisterEvent( Init, "EVENT_AVATAR_CREATED" )
	WidgetsInit()
	RegisterEvent( OnBuffsChanged,	"EVENT_OBJECT_BUFFS_CHANGED" )	
	RegisterEvent( SecondTimer,		"EVENT_SECOND_TIMER" )
	RegisterReactionHandler( OnReactionExecute, "execute" )
	--- "AddonManager"
	RegisterEvent( OnAMAddonInfoRequest,	"SCRIPT_ADDON_INFO_REQUEST" )
	RegisterEvent( OnAMAddonMemRequest,		"SCRIPT_ADDON_MEM_REQUEST" )
	--
	OnBuffsChanged()
end

local function ToggleDNDHandler(params)
	if params.target == common.GetAddonName() then
		DnD:Enable(wtMainPanel, params.state)
		DnD:Enable(wtAdditionalPanel, params.state)
	end
end
--------------------------------------------------------------------------------
if avatar.IsExist() then Init() else RegisterEvent( Init, "EVENT_AVATAR_CREATED" ) end
--------------------------------------------------------------------------------
